<?php

if (!defined("ABSPATH")) {
    exit();
}

class Paytree_WooCommerce_Gateway extends WC_Payment_Gateway
{
    public $api_token;
    public $api_endpoint;
    public $payment_methods;

    public function __construct()
    {
        $this->id = "paytree";
        $this->icon = "";
        $this->has_fields = true;
        $this->method_title = "Paytree";
        $this->method_description =
            "Integration with Paytree API supporting multiple payment methods and gateways.";

        // Load the settings
        $this->init_form_fields();
        $this->init_settings();

        // Logger
        $this->logger = wc_get_logger();

        // Define user settings
        $this->title = $this->get_option("title");
        $this->description = $this->get_option("description");
        $this->enabled = $this->get_option("enabled");
        $this->api_token = $this->get_option("api_token");
        $this->api_endpoint = $this->get_option("api_endpoint");
        $this->card_min_amount = $this->get_option("card_min_amount");
        $this->card_max_amount = $this->get_option("card_max_amount");
        $this->crypto_min_amount = $this->get_option("crypto_min_amount");
        $this->crypto_max_amount = $this->get_option("crypto_max_amount");
        $this->wire_min_amount = $this->get_option("wire_min_amount");
        $this->wire_max_amount = $this->get_option("wire_max_amount");
        $this->enable_card_method = $this->get_option("enable_card_method");
        $this->enable_crypto_method = $this->get_option("enable_crypto_method");
        $this->enable_wire_method = $this->get_option("enable_wire_method");
        $this->enable_log_request_body = $this->get_option("log_request_body");
        $this->enable_log_request_resp = $this->get_option("log_request_resp");
        // $this->payment_methods = array_filter($this->get_option('payment_methods'));

        // Add actions for saving admin settings and handling payment
        add_action("woocommerce_update_options_payment_gateways_" . $this->id, [
            $this,
            "process_admin_options",
        ]);
    }

    public function init_form_fields()
    {
        $this->form_fields = [
            "section_1_title" => [
                "type" => "title",
                "title" => "General",
                "description" =>
                    "Configure generic settings for Paytree below.",
            ],
            "enabled" => [
                "title" => "Enable/Disable",
                "label" => "Enable Paytree",
                "type" => "checkbox",
                "default" => "no",
                "description" => "Enable Paytree for processing payments.",
            ],
            "title" => [
                "title" => "Title",
                "type" => "text",
                "default" => "Online Payments",
                "description" => "Heading shoppers will see on checkout page.",
            ],
            "description" => [
                "title" => "Description",
                "type" => "textarea",
                "default" =>
                    "Pay securely using card, crypto, or wire transfer.",
                "description" =>
                    "Description shoppers will see on checkout page (Under heading).",
            ],
            "section_2_title" => [
                "type" => "title",
                "title" => "API Settings",
                "description" => "Configure connectivity settings below.",
            ],
            "api_endpoint" => [
                "title" => "API Endpoint",
                "type" => "text",
                "default" =>
                    "https://api.domain.com/v1/transaction/payment_intent/",
                "description" => "The API endpoint.",
            ],
            "api_token" => [
                "title" => "API Token",
                "type" => "text",
                "default" => "",
                "description" => "API token (Available in back office)",
            ],
            "section_3_title" => [
                "type" => "title",
                "title" => "Payment Methods",
                "description" =>
                    "Configure and toggle between different payment methods.",
            ],
            "enable_card_method" => [
                "title" => "Enable Card Payments",
                "type" => "checkbox",
                "label" => "Enable Card Payments",
                "default" => "yes",
            ],
            "enable_crypto_method" => [
                "title" => "Enable Crypto Payments",
                "type" => "checkbox",
                "label" => "Enable Crypto Payments",
                "default" => "no",
            ],
            "enable_wire_method" => [
                "title" => "Enable Wire Payments",
                "type" => "checkbox",
                "label" => "Enable Wire Payments",
                "default" => "no",
            ],
            "section_4_title" => [
                "type" => "title",
                "title" => "Payment Methods Limits",
                "description" =>
                    "Configure limits for each individual payment method.",
            ],
            "card_min_amount" => [
                "title" => "Min Card Amount",
                "type" => "number",
                "default" => "10",
                "description" => "Minimum amount to process card payments.",
            ],
            "card_max_amount" => [
                "title" => "Max Card Amount",
                "type" => "number",
                "default" => "500",
                "description" => "Maximum amount to process card payments.",
            ],
            "crypto_min_amount" => [
                "title" => "Min Crypto Amount",
                "type" => "number",
                "default" => "10",
                "description" => "Minimum amount to process crypto payments.",
            ],
            "crypto_max_amount" => [
                "title" => "Max Crypto Amount",
                "type" => "number",
                "default" => "500",
                "description" => "Maximum amount to process crypto payments.",
            ],
            "wire_min_amount" => [
                "title" => "Min Wire Amount",
                "type" => "number",
                "default" => "10",
                "description" => "Minimum amount to process wire payments.",
            ],
            "wire_max_amount" => [
                "title" => "Max Wire Amount",
                "type" => "number",
                "default" => "500",
                "description" => "Maximum amount to process wire payments.",
            ],
            "section_5_title" => [
                "type" => "title",
                "title" => "Developers",
                "description" => "Advanced for configuration and debugging.",
            ],
            "log_request_body" => [
                "title" => "Log API payload",
                "label" => "Enable",
                "type" => "checkbox",
                "default" => "no",
                "description" => "Log all API requests payload/body",
            ],
            "log_request_resp" => [
                "title" => "Log API response",
                "label" => "Enable",
                "type" => "checkbox",
                "default" => "no",
                "description" => "Log all API responses",
            ],
        ];
    }

    public function payment_fields()
    {
        // Display instructions or description
        if ($this->get_option("description")) {
            echo wpautop(wp_kses_post($this->get_option("description")));
        }

        // Display payment method options (Card, Crypto, Wire)
        echo '<div class="paytree-payment-methods">';

        if ("yes" === $this->get_option("enable_card_method")) {
            echo '<p class="form">
                    <input type="radio" id="paytree_card" name="paytree_method" value="card" required />
                    <label for="paytree_card">' .
                __("Pay with Card", "paytree") .
                '</label>
                  </p>';
        }

        if ("yes" === $this->get_option("enable_crypto_method")) {
            echo '<p class="form">
                    <input type="radio" id="paytree_crypto" name="paytree_method" value="crypto" required />
                    <label for="paytree_crypto">' .
                __("Pay with Crypto", "paytree") .
                '</label>
                  </p>';
        }

        if ("yes" === $this->get_option("enable_wire_method")) {
            echo '<p class="form">
                    <input type="radio" id="paytree_wire" name="paytree_method" value="wire" required />
                    <label for="paytree_wire">' .
                __("Pay with Wire Transfer", "paytree") .
                '</label>
                  </p>';
        }

        echo "</div>";
    }

    public function process_payment($order_id)
    {
        global $woocommerce;

        $order = wc_get_order($order_id);
        $total = $order->get_total();
        $currency = get_woocommerce_currency_symbol();
        $selected_method = sanitize_text_field($_POST["paytree_method"]);

        if ($selected_method == "card") {
            $min_amount = $this->card_min_amount;
            $max_amount = $this->card_max_amount;
        } elseif ($selected_method == "wire") {
            $min_amount = $this->wire_min_amount;
            $max_amount = $this->wire_max_amount;
        } elseif ($selected_method == "crypto") {
            $min_amount = $this->crypto_min_amount;
            $max_amount = $this->crypto_max_amount;
        }

        if (empty($selected_method)) {
            // Handle errors and show message to customer
            wc_add_notice(
                "Please select payment method to proceed checkout",
                "error"
            );
            return;
        }

        if ($total > $max_amount || $total < $min_amount) {
            // Handle errors and show message to customer
            wc_add_notice(
                "Minimum amount for proceed checkout is " .
                    $currency .
                    $min_amount .
                    " And Maximum amount is " .
                    $currency .
                    $max_amount .
                    " ",
                "error"
            );
            return;
        }

        // Step 1: Create Payment Intent with Paytree API
        $response = $this->create_payment_intent($order_id, $selected_method);

        if ($response && isset($response->payment_link)) {
            // Step 2: Redirect customer to payment page
            return [
                "result" => "success",
                "redirect" => $response->payment_link,
            ];
        } else {
            // Handle errors and show message to customer
            if (
                property_exists($response, "message") &&
                isset($response->message)
            ) {
                wc_add_notice("Payment error: " . $response->message, "error");
            } else {
                wc_add_notice(
                    "Payment gateway returned an error. Please try again or contact us if the issue persists.",
                    "error"
                );
            }

            // Log errors
            $this->logger->error("Failed to generate payment link.", [
                "source" => "woo-paytree",
                "backtrace" => false,
                "order_id" => $order_id,
                "response" => $response,
            ]);

            return;
        }
    }

    private function create_payment_intent($order_id, $payment_method)
    {
        $api_token = $this->api_token;
        $api_endpoint = $this->api_endpoint;
        $order = wc_get_order($order_id);
        $pay_uni_id = rand(0, 9999);
        $pay_uni_id = $order->get_order_key() . $pay_uni_id;

        update_option("paytree_transaction_ref", $order->get_order_number());
        // Prepare data to send to Paytree API
        $data = [
            "transaction_ref" => $order->get_order_number(),
            "client_ref" => $pay_uni_id,
            "amount" => number_format($order->get_total(), 2, ".", ""), // Convert amount to the required format
            "amount_currency" => $order->get_currency(),
            "method" => $payment_method,
            "customer" => [
                "first_name" => $order->get_billing_first_name(),
                "last_name" => $order->get_billing_last_name(),
                "email" => $order->get_billing_email(),
                "phone" => $order->get_billing_phone(),
            ],
            "address" => [
                "street" => $order->get_billing_address_1()
                    ? $order->get_billing_address_1()
                    : $order->get_shipping_address_1(),
                "city" => $order->get_billing_city()
                    ? $order->get_billing_city()
                    : $order->get_shipping_city(),
                "state" => $order->get_billing_state()
                    ? $order->get_billing_state()
                    : $order->get_shipping_state(),
                "zip" => $order->get_billing_postcode()
                    ? $order->get_billing_postcode()
                    : $order->get_shipping_postcode(),
                "country" => $order->get_billing_country()
                    ? $order->get_billing_country()
                    : $order->get_shipping_country(),
            ],
            "session" => [
                "ip_address" => $_SERVER["REMOTE_ADDR"],
                "user_agent" => $_SERVER["HTTP_USER_AGENT"],
            ],
            "callback" => [
                "notification_url" => site_url("/wp-json/paytree/v1/webhook/"),
                "return_url" => $this->get_return_url($order),
                "cancel_url" => wc_get_cart_url(), // Return to cart if canceled
            ],
        ];

        // Define the fields to check for emptiness
        $fields_to_check = [
            "address" => ["street", "city", "state", "zip", "country"],
        ];

        // Loop through each field in the 'address' array and check if it's empty
        foreach ($fields_to_check["address"] as $field) {
            if (empty($data["address"][$field])) {
                unset($data["address"][$field]);
            }
        }

        $response = wp_remote_post($api_endpoint, [
            "method" => "POST",
            "headers" => [
                "Authorization" => "Token " . $api_token,
                "Content-Type" => "application/json",
            ],
            "body" => json_encode($data),
        ]);

        if ($this->enable_log_request_body === "yes") {
            $this->logger->debug("API request body/payload", [
                "source" => "woo-paytree",
                "backtrace" => false,
                "endpoint" => $api_endpoint,
                "order_id" => $order_id,
                "data" => $data,
            ]);
        }

        if ($this->enable_log_request_resp === "yes") {
            $this->logger->debug("API request response", [
                "source" => "woo-paytree",
                "backtrace" => false,
                "endpoint" => $api_endpoint,
                "order_id" => $order_id,
                "response" => json_decode(wp_remote_retrieve_body($response)),
            ]);
        }

        if (is_wp_error($response)) {
            return false;
        }
        return json_decode(wp_remote_retrieve_body($response));
    }
}
