<?php
/**
 * Plugin Name: Paytree
 * Plugin URI: https://paytree.tech
 * Description: A WooCommerce payment gateway plugin that integrates with Paytree API.
 * Version:     1.0.3
 * Author:      Paytree
 * Author URI:  https://paytree.tech
 */

// Exit if accessed directly
if (!defined("ABSPATH")) {
    exit();
}

// Force Load WooCommerce Classes
if (!class_exists("WC_Payment_Gateway")) {
    include_once WP_PLUGIN_DIR .
        "/woocommerce/includes/abstracts/abstract-wc-settings-api.php";
    include_once WP_PLUGIN_DIR .
        "/woocommerce/includes/abstracts/abstract-wc-payment-gateway.php";
}

add_action("plugins_loaded", "paytree_init_gateway_class");
function paytree_init_gateway_class()
{
    if (!class_exists("WC_Payment_Gateway")) {
        return; // WooCommerce is not available
    }
    require_once plugin_dir_path(__FILE__) .
        "includes/class-paytree-woocommerce.php";
}

// Add Paytree Payment Gateway to WooCommerce
add_filter("woocommerce_payment_gateways", "add_paytree_gateway");

function add_paytree_gateway($gateways)
{
    $gateways[] = "Paytree_WooCommerce_Gateway";
    return $gateways;
}

// Activation and deactivation hooks
register_activation_hook(__FILE__, "paytree_activate");
register_deactivation_hook(__FILE__, "paytree_deactivate");

function paytree_activate()
{
    // Actions to perform during plugin activation, such as adding default settings
}

function paytree_deactivate()
{
    // Actions to perform during plugin deactivation
}

// Add webhook handler
add_action("rest_api_init", function () {
    register_rest_route("paytree/v1", "/webhook/", [
        "methods" => "GET",
        "callback" => "handle_paytree_webhook",
        "permission_callback" => "__return_true",
    ]);
});

function handle_paytree_webhook(WP_REST_Request $request)
{
    $gateway_settings = get_option("woocommerce_paytree_settings");
    $api_token = $gateway_settings["api_token"];
    $api_endpoint = $gateway_settings["api_endpoint"];

    // Refund or partial redund Order
    if (isset($_GET["webhook"]) && $_GET["webhook"] === "refund_order") {
        $gatway_api_token = isset($_GET["gateway_api_token"])
            ? sanitize_text_field($_GET["gateway_api_token"])
            : null;
        $refund_order_id = isset($_GET["wc_order_id"])
            ? sanitize_text_field($_GET["wc_order_id"])
            : null;
        $refund_amount = isset($_GET["refund_amount"])
            ? floatval($_GET["refund_amount"])
            : null;

        //Check Token is available or not
        if (
            !isset($_GET["gateway_api_token"]) ||
            $_GET["gateway_api_token"] !== $gatway_api_token
        ) {
            wp_send_json_success("Required Api token.");
        }
        //Check Order id is available or not
        if (
            !isset($_GET["wc_order_id"]) ||
            $_GET["wc_order_id"] !== $refund_order_id
        ) {
            wp_send_json_success("Required Order Id.");
        }
        //Check Refund Amount is available or not
        if (
            !isset($_GET["refund_amount"]) ||
            floatval($_GET["refund_amount"]) !== $refund_amount
        ) {
            wp_send_json_success("Required Refund Amount.");
        }
        if ($gatway_api_token !== $api_token) {
            wp_send_json_success("Invalid API Token.");
        }

        if ($refund_order_id && $refund_amount !== null) {
            $order = wc_get_order($refund_order_id);

            if ($order) {
                // Check if refund is full or partial
                if ($refund_amount >= $order->get_total()) {
                    // Full refund
                    $order->update_status("refunded");
                } else {
                    // Partial refund
                    $order->update_status("partially-refunded");
                }

                // Loop through each item in the order and restock
                foreach ($order->get_items() as $item_id => $item) {
                    $product = $item->get_product();
                    $quantity = $item->get_quantity();

                    if ($product && $product->managing_stock()) {
                        wc_update_product_stock(
                            $product,
                            $product->get_stock_quantity() + $quantity
                        );
                    }
                }

                $order->add_order_note(
                    "Order refunded with inventory restocked. Amount: $refund_amount"
                );
                wp_send_json_success("Order status updated successfully.");
            } else {
                wp_send_json_error("Order not found.");
            }
        } else {
            wp_send_json_error("Invalid data.");
        }
    }

    $transaction_ref = get_option("paytree_transaction_ref");
    $api_endpoint = $api_endpoint . "?id=" . $transaction_ref;
    $payment_status = "failed";

    $response = wp_remote_post($api_endpoint, [
        "method" => "GET",
        "headers" => [
            "Authorization" => "Token " . $api_token,
        ],
    ]);

    if (is_wp_error($response)) {
        $error_message = $response->get_error_message();
        return new WP_REST_Response(
            ["error" => "API request failed: " . $error_message],
            500
        );
    }

    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body, true);

    if (isset($data["results"]) && is_array($data["results"])) {
        foreach ($data["results"] as $transaction) {
            // Match the ID from API response with the ID you passed
            if ($transaction["transaction_ref"] == $transaction_ref) {
                // ID matches, now check the payment status
                $payment_status = $transaction["payment"][0]["status"];
                $order_id = $transaction["transaction_ref"];
                break;
            }
        }
    }

    // Fetch order based on $order_id
    $order = wc_get_order($order_id);

    // Update payment status in WooCommerce
    if ($payment_status === "success") {
        $order->payment_complete();
    } elseif ($payment_status === "pending") {
        $order->update_status("pending", "Payment pending via Paytree");
    } elseif ($payment_status === "partial") {
        $order->update_status("partial", "Payment partial via Paytree");
    } elseif ($payment_status === "cancelled") {
        $order->update_status("cancelled", "Payment cancelled via Paytree");
    } elseif ($payment_status === "expired") {
        $order->update_status("expired", "Payment expired via Paytree");
    } elseif ($payment_status === "authorized") {
        $order->update_status("authorized", "Payment authorized via Paytree");
    } elseif ($payment_status === "refunded") {
        $order->update_status("refunded", "Payment refunded via Paytree");
    } elseif ($payment_status === "partial_refunded") {
        $order->update_status(
            "partial_refunded",
            "Payment partial refunded via Paytree"
        );
    } elseif ($payment_status === "failed") {
        $order->update_status("failed", "Payment failed via Paytree");
    }

    return new WP_REST_Response("OK", 200);
}
